@@ -3,12 +3,12 @@ Somewhat hacky solution to create conda lock files.
 """
 
 import datetime
+import importlib.util
 import logging
 import os
 import pathlib
 import posixpath
 import re
-import subprocess
 import sys
 import tempfile
 
@@ -34,7 +34,7 @@ import click
 import pkg_resources
 import yaml
 
-from ensureconda import ensureconda
+from ensureconda.api import ensureconda
 from typing_extensions import Literal
 
 from conda_lock.click_helpers import OrderedGroup
@@ -62,12 +62,17 @@ try:
     PIP_SUPPORT = True
 except ImportError:
     PIP_SUPPORT = False
+from conda_lock.lookup import set_lookup_location
 from conda_lock.src_parser import (
     Dependency,
+    GitMeta,
+    InputMeta,
     LockedDependency,
     Lockfile,
     LockMeta,
     LockSpecification,
+    MetadataOption,
+    TimeMeta,
     UpdateSpecification,
     aggregate_lock_specs,
 )
@@ -185,7 +190,11 @@ def do_validate_platform(lockfile: str) -> None:
 
 
 def do_conda_install(
-    conda: PathLike, prefix: str, name: str, file: pathlib.Path
+    conda: PathLike,
+    prefix: "str | None",
+    name: "str | None",
+    file: pathlib.Path,
+    copy: bool,
 ) -> None:
 
     _conda = partial(_invoke_conda, conda, prefix, name, check_call=True)
@@ -202,13 +211,18 @@ def do_conda_install(
     else:
         pip_requirements = []
 
+    env_prefix = ["env"] if kind == "env" and not is_micromamba(conda) else []
+    copy_arg = ["--copy"] if kind != "env" and copy else []
+    yes_arg = ["--yes"] if kind != "env" else []
+
     _conda(
         [
-            *(["env"] if kind == "env" and not is_micromamba(conda) else []),
+            *env_prefix,
             "create",
+            *copy_arg,
             "--file",
             str(file),
-            *([] if kind == "env" else ["--yes"]),
+            *yes_arg,
         ],
     )
 
@@ -239,7 +253,7 @@ def make_lock_spec(
 ) -> LockSpecification:
     """Generate the lockfile specs from a set of input src_files.  If required_categories is set filter out specs that do not match those"""
     lock_specs = parse_source_files(
-        src_files=src_files, platform_overrides=platform_overrides or DEFAULT_PLATFORMS
+        src_files=src_files, platform_overrides=platform_overrides
     )
 
     lock_spec = aggregate_lock_specs(lock_specs)
@@ -282,6 +296,8 @@ def make_lock_files(
     filter_categories: bool = True,
     extras: Optional[AbstractSet[str]] = None,
     check_input_hash: bool = False,
+    metadata_choices: AbstractSet[MetadataOption] = frozenset(),
+    metadata_yamls: Sequence[pathlib.Path] = (),
 ) -> None:
     """
     Generate a lock file from the src files provided
@@ -315,6 +331,10 @@ def make_lock_files(
         Filter out unused categories prior to solving
     check_input_hash :
         Do not re-solve for each target platform for which specifications are unchanged
+    metadata_choices:
+        Set of selected metadata fields to generate for this lockfile.
+    metadata_yamls:
+        YAML or JSON file(s) containing structured metadata to add to metadata section of the lockfile.
     """
 
     # initialize virtual package fake
@@ -391,10 +411,16 @@ def make_lock_files(
                 platforms=platforms_to_lock,
                 lockfile_path=lockfile_path,
                 update_spec=update_spec,
+                metadata_choices=metadata_choices,
+                metadata_yamls=metadata_yamls,
             )
 
             if "lock" in kinds:
-                write_conda_lock_file(lock_content, lockfile_path)
+                write_conda_lock_file(
+                    lock_content,
+                    lockfile_path,
+                    metadata_choices=metadata_choices,
+                )
                 print(
                     " - Install lock using:",
                     KIND_USE_TEXT["lock"].format(lockfile=str(lockfile_path)),
@@ -420,6 +446,7 @@ def do_render(
     filename_template: Optional[str] = None,
     extras: Optional[AbstractSet[str]] = None,
     check_input_hash: bool = False,
+    override_platform: Optional[Sequence[str]] = None,
 ) -> None:
     """Render the lock content for each platform in lockfile
 
@@ -437,9 +464,12 @@ def do_render(
         Include the given extras in output
     check_input_hash :
         Do not re-render if specifications are unchanged
-
+    override_platform :
+        Generate only this subset of the platform files
     """
     platforms = lockfile.metadata.platforms
+    if override_platform is not None and len(override_platform) > 0:
+        platforms = list(sorted(set(platforms) & set(override_platform)))
 
     if filename_template:
         if "{platform}" not in filename_template and len(platforms) > 1:
@@ -553,8 +583,12 @@ def render_lockfile_for_platform(  # noqa: C901
         *(["dev"] if include_dev_dependencies else []),
     }
 
-    conda_deps = []
-    pip_deps = []
+    conda_deps: List[LockedDependency] = []
+    pip_deps: List[LockedDependency] = []
+
+    # ensure consistent ordering of generated file
+    lockfile.toposort_inplace()
+
     for p in lockfile.package:
         if p.platform == platform and ((not p.optional) or (p.category in categories)):
             if p.manager == "pip":
@@ -570,9 +604,15 @@ def render_lockfile_for_platform(  # noqa: C901
         if spec.source and spec.source.type == "url":
             return f"{spec.name} @ {spec.source.url}"
         elif direct:
-            return f"{spec.name} @ {spec.url}#md5={spec.hash.md5}"
+            s = f"{spec.name} @ {spec.url}"
+            if spec.hash.sha256:
+                s += f"#sha256={spec.hash.sha256}"
+            return s
         else:
-            return f"{spec.name} === {spec.version} --hash=md5:{spec.hash.md5}"
+            s = f"{spec.name} === {spec.version}"
+            if spec.hash.sha256:
+                s += f" --hash=sha256:{spec.hash.sha256}"
+            return s
 
     def format_conda_requirement(
         spec: LockedDependency, platform: str, direct: bool = False
@@ -636,6 +676,15 @@ def render_lockfile_for_platform(  # noqa: C901
                 for dep in pip_deps
             ]
         )
+
+        if len(pip_deps) > 0:
+            logger.warning(
+                "WARNING: installation of pip dependencies is only supported "
+                "by the 'conda-lock install' command. Other tools may silently "
+                "ignore them. For portability, we recommend using the newer "
+                "unified lockfile format (i.e. removing the --kind=explicit "
+                "argument."
+            )
     else:
         raise ValueError(f"Unrecognised lock kind {kind}.")
 
@@ -701,6 +750,34 @@ def _solve_for_arch(
     return list(conda_deps.values()) + list(pip_deps.values())
 
 
+def convert_structured_metadata_yaml(in_path: pathlib.Path) -> Dict[str, Any]:
+    with in_path.open("r") as infile:
+        metadata = yaml.safe_load(infile)
+    return metadata
+
+
+def update_metadata(to_change: Dict[str, Any], change_source: Dict[str, Any]) -> None:
+    for key in change_source:
+        if key in to_change:
+            logger.warning(
+                f"Custom metadata field {key} provided twice, overwriting value "
+                + f"{to_change[key]} with {change_source[key]}"
+            )
+    to_change.update(change_source)
+
+
+def get_custom_metadata(
+    metadata_yamls: Sequence[pathlib.Path],
+) -> Optional[Dict[str, str]]:
+    custom_metadata_dict: Dict[str, str] = {}
+    for yaml_path in metadata_yamls:
+        new_metadata = convert_structured_metadata_yaml(yaml_path)
+        update_metadata(custom_metadata_dict, new_metadata)
+    if custom_metadata_dict:
+        return custom_metadata_dict
+    return None
+
+
 def create_lockfile_from_spec(
     *,
     conda: PathLike,
@@ -708,6 +785,8 @@ def create_lockfile_from_spec(
     platforms: List[str] = [],
     lockfile_path: pathlib.Path,
     update_spec: Optional[UpdateSpecification] = None,
+    metadata_choices: AbstractSet[MetadataOption] = frozenset(),
+    metadata_yamls: Sequence[pathlib.Path] = (),
 ) -> Lockfile:
     """
     Solve or update specification
@@ -730,22 +809,67 @@ def create_lockfile_from_spec(
         for dep in deps:
             locked[(dep.manager, dep.name, dep.platform)] = dep
 
+    spec_sources: Dict[str, pathlib.Path] = {}
+    for source in spec.sources:
+        try:
+            path = relative_path(lockfile_path.parent, source)
+        except ValueError as e:
+            if "Paths don't have the same drive" not in str(e):
+                raise e
+            path = str(source.resolve())
+        spec_sources[path] = source
+
+    if MetadataOption.TimeStamp in metadata_choices:
+        time_metadata = TimeMeta.create()
+    else:
+        time_metadata = None
+
+    if metadata_choices & {
+        MetadataOption.GitUserEmail,
+        MetadataOption.GitUserName,
+        MetadataOption.GitSha,
+    }:
+        if not importlib.util.find_spec("git"):
+            raise RuntimeError(
+                "The GitPython package is required to read Git metadata."
+            )
+        git_metadata = GitMeta.create(
+            metadata_choices=metadata_choices,
+            src_files=spec.sources,
+        )
+    else:
+        git_metadata = None
+
+    if metadata_choices & {MetadataOption.InputSha, MetadataOption.InputMd5}:
+        inputs_metadata: Optional[Dict[str, InputMeta]] = {
+            relative_path: InputMeta.create(
+                metadata_choices=metadata_choices, src_file=src_file
+            )
+            for relative_path, src_file in spec_sources.items()
+        }
+    else:
+        inputs_metadata = None
+
+    custom_metadata = get_custom_metadata(metadata_yamls=metadata_yamls)
+
     return Lockfile(
         package=[locked[k] for k in locked],
         metadata=LockMeta(
             content_hash=spec.content_hash(),
             channels=[c for c in spec.channels],
             platforms=spec.platforms,
-            sources=[
-                relative_path(lockfile_path.parent, source) for source in spec.sources
-            ],
+            sources=[str(source.resolve()) for source in spec.sources],
+            git_metadata=git_metadata,
+            time_metadata=time_metadata,
+            inputs_metadata=inputs_metadata,
+            custom_metadata=custom_metadata,
         ),
     )
 
 
 def parse_source_files(
     src_files: List[pathlib.Path],
-    platform_overrides: Sequence[str],
+    platform_overrides: Optional[Sequence[str]],
 ) -> List[LockSpecification]:
     """
     Parse a sequence of dependency specifications from source files
@@ -755,19 +879,26 @@ def parse_source_files(
     src_files :
         Files to parse for dependencies
     platform_overrides :
-        Target platforms to render meta.yaml files for
+        Target platforms to render environment.yaml and meta.yaml files for
     """
-    desired_envs = []
+    desired_envs: List[LockSpecification] = []
     for src_file in src_files:
         if src_file.name == "meta.yaml":
             desired_envs.append(
-                parse_meta_yaml_file(src_file, list(platform_overrides))
+                parse_meta_yaml_file(
+                    src_file, list(platform_overrides or DEFAULT_PLATFORMS)
+                )
             )
         elif src_file.name == "pyproject.toml":
             desired_envs.append(parse_pyproject_toml(src_file))
         else:
             desired_envs.append(
-                parse_environment_file(src_file, pip_support=PIP_SUPPORT)
+                parse_environment_file(
+                    src_file,
+                    platform_overrides,
+                    default_platforms=DEFAULT_PLATFORMS,
+                    pip_support=PIP_SUPPORT,
+                )
             )
     return desired_envs
 
@@ -859,8 +990,22 @@ def _render_lockfile_for_install(
 
     platform = platform_subdir()
     if platform not in lock_content.metadata.platforms:
+        suggested_platforms_section = "platforms:\n- "
+        suggested_platforms_section += "\n- ".join(
+            [platform] + lock_content.metadata.platforms
+        )
+        suggested_platform_args = "--platform=" + " --platform=".join(
+            [platform] + lock_content.metadata.platforms
+        )
         raise PlatformValidationError(
-            f"Dependencies are not locked for the current platform ({platform})"
+            f"The lockfile {filename} does not contain a solution for the current "
+            f"platform {platform}. The lockfile only contains solutions for the "
+            f"following platforms: {', '.join(lock_content.metadata.platforms)}. In "
+            f"order to add support for {platform}, you must regenerate the lockfile. "
+            f"Either add the following section to your environment.yml:\n\n"
+            f"{suggested_platforms_section}\n\n"
+            f"or add the following arguments to the conda-lock command:\n\n"
+            f"{suggested_platform_args}\n\n"
         )
 
     # TODO: Move to LockFile
@@ -903,6 +1048,8 @@ def run_lock(
     virtual_package_spec: Optional[pathlib.Path] = None,
     update: Optional[List[str]] = None,
     filter_categories: bool = False,
+    metadata_choices: AbstractSet[MetadataOption] = frozenset(),
+    metadata_yamls: Sequence[pathlib.Path] = (),
 ) -> None:
     if environment_files == DEFAULT_FILES:
         if lockfile_path.exists():
@@ -947,10 +1094,13 @@ def run_lock(
         extras=extras,
         check_input_hash=check_input_hash,
         filter_categories=filter_categories,
+        metadata_choices=metadata_choices,
+        metadata_yamls=metadata_yamls,
     )
 
 
 @click.group(cls=OrderedGroup, default="lock", default_if_no_args=True)
+@click.version_option()
 def main() -> None:
     """To get help for subcommands, use the conda-lock <SUBCOMMAND> --help"""
     pass
@@ -1072,10 +1222,34 @@ TLogLevel = Union[
     multiple=True,
     help="Packages to update to their latest versions. If empty, update all.",
 )
+@click.option(
+    "--pypi_to_conda_lookup_file",
+    type=str,
+    help="Location of the lookup file containing Pypi package names to conda names.",
+)
+@click.option(
+    "--md",
+    "--metadata",
+    "metadata_choices",
+    default=[],
+    multiple=True,
+    type=click.Choice([md.value for md in MetadataOption]),
+    help="Metadata fields to include in lock-file",
+)
+@click.option(
+    "--mdy",
+    "--metadata-yaml",
+    "--metadata-json",
+    "metadata_yamls",
+    default=[],
+    multiple=True,
+    type=click.Path(),
+    help="YAML or JSON file(s) containing structured metadata to add to metadata section of the lockfile.",
+)
 @click.pass_context
 def lock(
     ctx: click.Context,
-    conda: Optional[PathLike],
+    conda: Optional[str],
     mamba: bool,
     micromamba: bool,
     platform: List[str],
@@ -1091,8 +1265,11 @@ def lock(
     check_input_hash: bool,
     log_level: TLogLevel,
     pdb: bool,
-    virtual_package_spec: Optional[PathLike],
-    update: bool = None,
+    virtual_package_spec: Optional[pathlib.Path],
+    pypi_to_conda_lookup_file: Optional[str],
+    update: Optional[List[str]] = None,
+    metadata_choices: Sequence[str] = (),
+    metadata_yamls: Sequence[pathlib.Path] = (),
 ) -> None:
     """Generate fully reproducible lock files for conda environments.
 
@@ -1108,6 +1285,14 @@ def lock(
     """
     logging.basicConfig(level=log_level)
 
+    # Set Pypi <--> Conda lookup file location
+    if pypi_to_conda_lookup_file:
+        set_lookup_location(pypi_to_conda_lookup_file)
+
+    metadata_enum_choices = set(MetadataOption(md) for md in metadata_choices)
+
+    metadata_yamls = [pathlib.Path(path) for path in metadata_yamls]
+
     # bail out if we do not encounter the default file if no files were passed
     if ctx.get_parameter_source("files") == click.core.ParameterSource.DEFAULT:
         candidates = list(files)
@@ -1152,6 +1337,8 @@ def lock(
         virtual_package_spec=virtual_package_spec,
         update=update,
         filter_categories=filter_categories,
+        metadata_choices=metadata_enum_choices,
+        metadata_yamls=metadata_yamls,
     )
     if strip_auth:
         with tempfile.TemporaryDirectory() as tempdir:
@@ -1182,6 +1369,14 @@ def lock(
     default=False,
     help="don't attempt to use or install micromamba.",
 )
+@click.option(
+    "--copy",
+    is_flag=True,
+    help=(
+        "Install using `--copy` to prevent links. "
+        "This is useful for building containers"
+    ),
+)
 @click.option("-p", "--prefix", help="Full path to environment location (i.e. prefix).")
 @click.option("-n", "--name", help="Name of environment.")
 @click.option(
@@ -1223,6 +1418,7 @@ def install(
     conda: Optional[str],
     mamba: bool,
     micromamba: bool,
+    copy: bool,
     prefix: Optional[str],
     name: Optional[str],
     lock_file: pathlib.Path,
@@ -1245,7 +1441,9 @@ def install(
         yaml.safe_load(auth) if auth else read_json(auth_file) if auth_file else None
     )
     _conda_exe = determine_conda_executable(conda, mamba=mamba, micromamba=micromamba)
-    install_func = partial(do_conda_install, conda=_conda_exe, prefix=prefix, name=name)
+    install_func = partial(
+        do_conda_install, conda=_conda_exe, prefix=prefix, name=name, copy=copy
+    )
     if validate_platform and not lock_file.name.endswith(DEFAULT_LOCKFILE_NAME):
         lockfile_contents = read_file(lock_file)
         try:
@@ -1257,7 +1455,7 @@ def install(
     with _render_lockfile_for_install(
         lock_file, include_dev_dependencies=dev, extras=set(extras)
     ) as lockfile:
-        if auth:
+        if _auth is not None:
             with _add_auth(read_file(lockfile), _auth) as lockfile_with_auth:
                 install_func(file=lockfile_with_auth)
         else:
@@ -1301,6 +1499,12 @@ def install(
 @click.option(
     "--pdb", is_flag=True, help="Drop into a postmortem debugger if conda-lock crashes"
 )
+@click.option(
+    "-p",
+    "--platform",
+    multiple=True,
+    help="render lock files for the following platforms",
+)
 @click.argument("lock-file", default=DEFAULT_LOCKFILE_NAME)
 @click.pass_context
 def render(
@@ -1312,6 +1516,7 @@ def render(
     log_level: TLogLevel,
     lock_file: PathLike,
     pdb: bool,
+    platform: Sequence[str],
 ) -> None:
     """Render multi-platform lockfile into single-platform env or explicit file"""
     logging.basicConfig(level=log_level)
@@ -1333,6 +1538,7 @@ def render(
         kinds=kind,
         include_dev_dependencies=dev_dependencies,
         extras=set(extras),
+        override_platform=platform,
     )
 
 
