@@ -2,7 +2,6 @@ import json
 import logging
 import os
 import pathlib
-import re
 import shlex
 import subprocess
 import sys
@@ -24,6 +23,7 @@ from urllib.parse import urlsplit, urlunsplit
 
 from typing_extensions import TypedDict
 
+from conda_lock._vendor.conda.models.match_spec import MatchSpec
 from conda_lock.invoke_conda import (
     PathLike,
     _get_conda_flags,
@@ -39,7 +39,6 @@ from conda_lock.src_parser import (
     VersionedDependency,
     _apply_categories,
 )
-from conda_lock.vendor.conda.models.match_spec import MatchSpec
 
 
 logger = logging.getLogger(__name__)
@@ -88,7 +87,10 @@ class DryRunInstall(TypedDict):
 
 
 def _to_match_spec(
-    conda_dep_name: str, conda_version: Optional[str], build: Optional[str]
+    conda_dep_name: str,
+    conda_version: Optional[str],
+    build: Optional[str],
+    conda_channel: Optional[str],
 ) -> str:
     kwargs = dict(name=conda_dep_name)
     if conda_version:
@@ -97,10 +99,17 @@ def _to_match_spec(
         kwargs["build"] = build
         if "version" not in kwargs:
             kwargs["version"] = "*"
+    if conda_channel:
+        kwargs["channel"] = conda_channel
 
     ms = MatchSpec(**kwargs)
     # Since MatchSpec doesn't round trip to the cli well
-    return ms.conda_build_form()
+    if conda_channel:
+        # this will return "channel_name::package_name"
+        return str(ms)
+    else:
+        # this will return only "package_name" even if there's a channel in the kwargs
+        return ms.conda_build_form()
 
 
 def extract_json_object(proc_stdout: str) -> str:
@@ -139,7 +148,7 @@ def solve_conda(
     """
 
     conda_specs = [
-        _to_match_spec(dep.name, dep.version, dep.build)
+        _to_match_spec(dep.name, dep.version, dep.build, dep.conda_channel)
         for dep in specs.values()
         if isinstance(dep, VersionedDependency) and dep.manager == "conda"
     ]
@@ -410,7 +419,7 @@ def update_specs_for_arch(
                 )
             )
         }
-        spec_for_name = {MatchSpec(v).name: v for v in specs}
+        spec_for_name = {MatchSpec(v).name: v for v in specs}  # type: ignore
         to_update = [
             spec_for_name[name] for name in set(installed).intersection(update)
         ]
