@@ -7,7 +7,6 @@ import yaml
 
 from conda_lock.common import get_in
 from conda_lock.src_parser import Dependency, LockSpecification, aggregate_lock_specs
-from conda_lock.src_parser.pyproject_toml import parse_python_requirement
 from conda_lock.src_parser.selectors import filter_platform_selectors
 
 
@@ -33,7 +32,7 @@ class UndefinedNeverFail(jinja2.Undefined):
     def __init__(  # type: ignore
         self,
         hint=None,
-        obj=jinja2.runtime.missing,
+        obj=jinja2.utils.missing,
         name=None,
         exc=jinja2.exceptions.UndefinedError,
     ) -> None:
@@ -56,7 +55,7 @@ class UndefinedNeverFail(jinja2.Undefined):
         try:
             return object.__getattr__(self, k)  # type: ignore
         except AttributeError:
-            return self._return_undefined(self._undefined_name + "." + k)
+            return self._return_undefined(self._undefined_name + "." + k)  # type: ignore
 
     # Unlike the methods above, Python requires that these
     # few methods must always return the correct type
@@ -134,14 +133,10 @@ def _parse_meta_yaml_file_for_platform(
     def add_spec(spec: str, category: str) -> None:
         if spec is None:
             return
-        # TODO: This does not parse conda requirements with build strings
-        dep = parse_python_requirement(
-            spec,
-            manager="conda",
-            optional=category != "main",
-            category=category,
-            normalize_name=False,
-        )
+
+        from .conda_common import conda_spec_to_versioned_dep
+
+        dep = conda_spec_to_versioned_dep(spec, category)
         dep.selectors.platform = [platform]
         dependencies.append(dep)
 
