@@ -1,20 +1,26 @@
 import logging
 import re
 
-from typing import Iterator
+from typing import Iterator, Optional
 
 
 logger = logging.getLogger(__name__)
 
 
-def filter_platform_selectors(content: str, platform: str) -> Iterator[str]:
+def filter_platform_selectors(
+    content: str, platform: Optional[str] = None
+) -> Iterator[str]:
     """ """
     # we support a very limited set of selectors that adhere to platform only
+    # https://docs.conda.io/projects/conda-build/en/latest/resources/define-metadata.html#preprocessing-selectors
+
     platform_sel = {
         "linux-64": {"linux64", "unix", "linux"},
         "linux-aarch64": {"aarch64", "unix", "linux"},
         "linux-ppc64le": {"ppc64le", "unix", "linux"},
-        "osx-64": {"osx", "osx64", "unix"},
+        # "osx64" is a selector unique to conda-build referring to
+        # platforms on macOS and the Python architecture is x86-64
+        "osx-64": {"osx64", "osx", "unix"},
         "osx-arm64": {"arm64", "osx", "unix"},
         "win-64": {"win", "win64"},
     }
@@ -25,9 +31,9 @@ def filter_platform_selectors(content: str, platform: str) -> Iterator[str]:
         if line.lstrip().startswith("#"):
             continue
         m = sel_pat.match(line)
-        if m:
+        if platform and m:
             cond = m.group(3)
-            if platform and (cond in platform_sel[platform]):
+            if cond in platform_sel[platform]:
                 yield line
             else:
                 logger.warning(
